var canvas;
var gl;
var perspectiveMatrix;
var shaderProgram;
var vertexPositionAttribute;
var squareVerticesBuffer;

var rotation = 0.0;
var lastUpdateTime;


function start() {
    canvas = document.getElementById("glCanvas");

    initWebGL(canvas); // Initialisierung des WebGL Kontextes

    // Es geht nur weiter, wenn WebGl verfügbar ist.

    if (gl) {
        gl.clearColor(0.0, 0.0, 0.0, 1.0); // Setzt die Farbe auf Schwarz, vollständig sichtbar
        gl.clearDepth(1.0); // Lösche alles, um die neue Farbe sichtbar zu machen
        gl.enable(gl.DEPTH_TEST); // Aktiviere Tiefentest
        gl.depthFunc(gl.LEQUAL); // Nähere Objekte verdecken entferntere Objekte
        //gl.clear(gl.COLOR_BUFFER_BIT); //necessary to realy clear!
    }
    
    initShaders();
    
    initBuffers();
    
    setInterval(drawScene, 15);
}

function initWebGL() {
    gl = null;

    try {
        gl = canvas.getContext("experimental-webgl");
    } catch (e) {}

    // Wenn wir keinen WebGl Kontext haben

    if (!gl) {
        alert("WebGL konnte nicht initialisiert werden.");
    }
}

function initShaders() {
    var frag = getShader(gl, "fragment");
    var vert = getShader(gl, "vertex");

    shaderProgram = gl.createProgram();
    gl.attachShader(shaderProgram, vert);
    gl.attachShader(shaderProgram, frag);
    gl.linkProgram(shaderProgram);

    if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
        alert("Initialisierung des Shaderprogramms nicht möglich.");
    }

    gl.useProgram(shaderProgram);

    vertexPositionAttribute = gl.getAttribLocation(shaderProgram, "aVertexPosition");
    gl.enableVertexAttribArray(vertexPositionAttribute);
}

function getShader(gl, id) {
    var shaderScript = document.getElementById(id);

    if (!shaderScript) {
        return null;
    }

    var shader;

    if (shaderScript.type == "x-shader/x-fragment") {
        shader = gl.createShader(gl.FRAGMENT_SHADER);
    } else if (shaderScript.type == "x-shader/x-vertex") {
        shader = gl.createShader(gl.VERTEX_SHADER);
    } else {
        return null; // Unbekannter Shadertyp
    }

    var theSource = "";
    var currentChild = shaderScript.firstChild;
    

    //collect the elements source code in 'theSource'
    while (currentChild) {
        if (currentChild.nodeType == 3) {
            theSource += currentChild.textContent;
        }

        currentChild = currentChild.nextSibling;
    }

    gl.shaderSource(shader, theSource);
    gl.compileShader(shader);

    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
        alert("Es ist ein Fehler beim Kompilieren der Shader aufgetaucht: " + gl.getShaderInfoLog(shader));
        return null;
    }

    return shader;
}


function initBuffers() {
    squareVerticesBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, squareVerticesBuffer);
    
    var vertices = [
        1.0, 1.0, 0.0,
        -1.0, 1.0, 0.0,
        1.0, -1.0, 0.0,
        -1.0, -1.0, 0.0
    ];
    
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);
}

function drawScene() {
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
    
    perspectiveMatrix = makePerspective(45, canvas.width/canvas.height, 0.1, 100.0);
    
    loadIdentity();
    mvTranslate([-0.0, 0.0, -6.0]);
    //saves the actual model view matrix onto a stack, then rotates it and draws it.
    mvPushMatrix();
    mvRotate(rotation, [1, 0, 1]);
    
    gl.bindBuffer(gl.ARRAY_BUFFER, squareVerticesBuffer);
    gl.vertexAttribPointer(vertexPositionAttribute, 3, gl.FLOAT, false, 0, 0);
    setMatrixUniforms();
    gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);
    
    //afterwards restores the original model view matrix.
    mvPopMatrix();
    
    //adapt rotation according to passed time between redraws.
    var currentTime = (new Date).getTime();
    if(lastUpdateTime) {
        var delta = currentTime - lastUpdateTime;
        rotation += (30 * delta) / 1000.0;
    }
    lastUpdateTime = currentTime;
}

//Hilfsroutinen aus einem Beispiel von Vlad Vukićević
var mvMatrixStack = [];

function mvPushMatrix(m) {
  if (m) {
    mvMatrixStack.push(m.dup());
    mvMatrix = m.dup();
  } else {
    mvMatrixStack.push(mvMatrix.dup());
  }
}

function mvPopMatrix() {
  if (!mvMatrixStack.length) {
    throw("Can't pop from an empty matrix stack.");
  }
  
  mvMatrix = mvMatrixStack.pop();
  return mvMatrix;
}

function mvRotate(angle, v) {
  var inRadians = angle * Math.PI / 180.0;
  
  var m = Matrix.Rotation(inRadians, $V([v[0], v[1], v[2]])).ensure4x4();
  multMatrix(m);
}

//Matrix Hilfsoperationen
function loadIdentity() {
  mvMatrix = Matrix.I(4);
}

function multMatrix(m) {
  mvMatrix = mvMatrix.x(m);
}

function mvTranslate(v) {
  multMatrix(Matrix.Translation($V([v[0], v[1], v[2]])).ensure4x4());
}

function setMatrixUniforms() {
  var pUniform = gl.getUniformLocation(shaderProgram, "uPMatrix");
  gl.uniformMatrix4fv(pUniform, false, new Float32Array(perspectiveMatrix.flatten()));

  var mvUniform = gl.getUniformLocation(shaderProgram, "uMVMatrix");
  gl.uniformMatrix4fv(mvUniform, false, new Float32Array(mvMatrix.flatten()));
}